# Sync for Expenses

<!-- Start Codat Library Description -->
﻿Embedded accounting integrations for corporate card providers.
<!-- End Codat Library Description -->

<!-- Start Summary [summary] -->
## Summary

Sync for Expenses: The API for Sync for Expenses.

Sync for Expenses is an API and a set of supporting tools. It has been built to
enable corporate card and expense management platforms to provide high-quality
integrations with multiple accounting software through a standardized API.

[Explore product](https://docs.codat.io/sync-for-expenses/overview) | [See our OpenAPI spec](https://github.com/codatio/oas)

Not seeing the endpoints you're expecting? We've [reorganized our products](https://docs.codat.io/updates/230901-new-products), and you may be using a [different version of Sync for Expenses](https://docs.codat.io/sync-for-expenses-v1-api#/).

---
<!-- Start Codat Tags Table -->
## Endpoints

| Endpoints | Description |
| :- |:- |
| Companies | Create and manage your SMB users' companies. |
| Connections | Create new and manage existing data connections for a company. |
| Configuration | View and manage mapping configuration and defaults for expense transactions. |
| Sync | Monitor the status of data syncs. |
| Expenses | Create and update transactions that represent your customers' spend. |
| Transfers | Create and update transactions that represent the movement of your customers' money. |
| Reimbursements | Create and update transactions that represent your customers' repayable spend. |
| Attachments | Attach receipts to a transaction for a complete audit trail. |
| Transaction status | Monitor the status of individual transactions in data syncs. |
| Manage data | Control and monitor the retrieval of data from an integration. |
| Push operations | View historic push operations. |
| Accounts | Create accounts and view account schemas. |
| Customers | Get, create, and update customers. |
| Suppliers | Get, create, and update suppliers. |
<!-- End Codat Tags Table -->
<!-- End Summary [summary] -->

<!-- Start Table of Contents [toc] -->
## Table of Contents

* [SDK Installation](https://github.com/codatio/client-sdk-python/blob/master/#sdk-installation)
* [IDE Support](https://github.com/codatio/client-sdk-python/blob/master/#ide-support)
* [SDK Example Usage](https://github.com/codatio/client-sdk-python/blob/master/#sdk-example-usage)
* [Available Resources and Operations](https://github.com/codatio/client-sdk-python/blob/master/#available-resources-and-operations)
* [File uploads](https://github.com/codatio/client-sdk-python/blob/master/#file-uploads)
* [Retries](https://github.com/codatio/client-sdk-python/blob/master/#retries)
* [Error Handling](https://github.com/codatio/client-sdk-python/blob/master/#error-handling)
* [Server Selection](https://github.com/codatio/client-sdk-python/blob/master/#server-selection)
* [Custom HTTP Client](https://github.com/codatio/client-sdk-python/blob/master/#custom-http-client)
* [Authentication](https://github.com/codatio/client-sdk-python/blob/master/#authentication)
* [Debugging](https://github.com/codatio/client-sdk-python/blob/master/#debugging)
<!-- End Table of Contents [toc] -->

<!-- Start SDK Installation [installation] -->
## SDK Installation

The SDK can be installed with either *pip* or *poetry* package managers.

### PIP

*PIP* is the default package installer for Python, enabling easy installation and management of packages from PyPI via the command line.

```bash
pip install codat-sync-for-expenses
```

### Poetry

*Poetry* is a modern tool that simplifies dependency management and package publishing by using a single `pyproject.toml` file to handle project metadata and dependencies.

```bash
poetry add codat-sync-for-expenses
```
<!-- End SDK Installation [installation] -->

## Example Usage
<!-- Start IDE Support [idesupport] -->
## IDE Support

### PyCharm

Generally, the SDK will work well with most IDEs out of the box. However, when using PyCharm, you can enjoy much better integration with Pydantic by installing an additional plugin.

- [PyCharm Pydantic Plugin](https://docs.pydantic.dev/latest/integrations/pycharm/)
<!-- End IDE Support [idesupport] -->

<!-- Start SDK Example Usage [usage] -->
## SDK Example Usage

### Example

```python
# Synchronous Example
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared

with CodatSyncExpenses(
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.companies.create(request={
        "name": "Technicalium",
    })

    if res is not None:
        # handle response
        pass
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import asyncio
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared

async def main():
    async with CodatSyncExpenses(
        security=shared.Security(
            auth_header="Basic BASE_64_ENCODED(API_KEY)",
        ),
    ) as s:
        res = await s.companies.create_async(request={
            "name": "Technicalium",
        })

        if res is not None:
            # handle response
            pass

asyncio.run(main())
```
<!-- End SDK Example Usage [usage] -->

<!-- Start Available Resources and Operations [operations] -->
## Available Resources and Operations

<details open>
<summary>Available methods</summary>

### [accounts](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/accounts/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/accounts/README.md#create) - Create account
* [get_create_model](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/accounts/README.md#get_create_model) - Get create account model

### [adjustments](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/adjustments/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/adjustments/README.md#create) - Create adjustment transaction

### [attachments](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/attachments/README.md)

* [upload](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/attachments/README.md#upload) - Upload attachment

### [bank_accounts](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/bankaccounts/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/bankaccounts/README.md#create) - Create bank account
* [get_create_model](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/bankaccounts/README.md#get_create_model) - Get create bank account model


### [companies](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md#create) - Create company
* [delete](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md#delete) - Delete a company
* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md#get) - Get company
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md#list) - List companies
* [update](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companies/README.md#update) - Update company

### [company_info](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companyinfo/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/companyinfo/README.md#get) - Get company info

### [configuration](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/configuration/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/configuration/README.md#get) - Get company configuration
* [set](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/configuration/README.md#set) - Set company configuration

### [connections](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#create) - Create connection
* [create_partner_expense_connection](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#create_partner_expense_connection) - Create partner expense connection
* [delete](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#delete) - Delete connection
* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#get) - Get connection
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#list) - List connections
* [unlink](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/connections/README.md#unlink) - Unlink connection

### [customers](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/customers/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/customers/README.md#create) - Create customer
* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/customers/README.md#get) - Get customer
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/customers/README.md#list) - List customers
* [update](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/customers/README.md#update) - Update customer

### [expenses](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/expenses/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/expenses/README.md#create) - Create expense transaction
* [update](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/expenses/README.md#update) - Update expense transactions

### [manage_data](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md#get) - Get data status
* [get_pull_operation](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md#get_pull_operation) - Get pull operation
* [list_pull_operations](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md#list_pull_operations) - List pull operations
* [refresh_all_data_types](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md#refresh_all_data_types) - Refresh all data
* [refresh_data_type](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/managedata/README.md#refresh_data_type) - Refresh data type

### [mapping_options](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/mappingoptions/README.md)

* [get_mapping_options](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/mappingoptions/README.md#get_mapping_options) - Mapping options

### [push_operations](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/pushoperations/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/pushoperations/README.md#get) - Get push operation
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/pushoperations/README.md#list) - List push operations

### [reimbursements](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/reimbursements/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/reimbursements/README.md#create) - Create reimbursable expense transaction
* [update](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/reimbursements/README.md#update) - Update reimbursable expense transaction

### [suppliers](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/suppliers/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/suppliers/README.md#create) - Create supplier
* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/suppliers/README.md#get) - Get supplier
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/suppliers/README.md#list) - List suppliers
* [update](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/suppliers/README.md#update) - Update supplier

### [sync](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/sync/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/sync/README.md#get) - Get sync status
* [get_last_successful_sync](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/sync/README.md#get_last_successful_sync) - Last successful sync
* [get_latest_sync](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/sync/README.md#get_latest_sync) - Latest sync status
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/sync/README.md#list) - List sync statuses

### [transaction_status](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/transactionstatus/README.md)

* [get](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/transactionstatus/README.md#get) - Get sync transaction
* [list](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/transactionstatus/README.md#list) - List sync transactions

### [transfers](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/transfers/README.md)

* [create](https://github.com/codatio/client-sdk-python/blob/master/docs/sdks/transfers/README.md#create) - Create transfer transaction

</details>
<!-- End Available Resources and Operations [operations] -->



<!-- Start File uploads [file-upload] -->
## File uploads

Certain SDK methods accept file objects as part of a request body or multi-part request. It is possible and typically recommended to upload files as a stream rather than reading the entire contents into memory. This avoids excessive memory consumption and potentially crashing with out-of-memory errors when working with very large files. The following example demonstrates how to attach a file stream to a request.

> [!TIP]
>
> For endpoints that handle file uploads bytes arrays can also be used. However, using streams is recommended for large files.
>

```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared

with CodatSyncExpenses(
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.attachments.upload(request={
        "company_id": "8a210b68-6988-11ed-a1eb-0242ac120002",
        "sync_id": "6fb40d5e-b13e-11ed-afa1-0242ac120002",
        "transaction_id": "336694d8-2dca-4cb5-a28d-3ccb83e55eee",
    })

    if res is not None:
        # handle response
        pass

```
<!-- End File uploads [file-upload] -->

<!-- Start Retries [retries] -->
## Retries

Some of the endpoints in this SDK support retries. If you use the SDK without any configuration, it will fall back to the default retry strategy provided by the API. However, the default retry strategy can be overridden on a per-operation basis, or across the entire SDK.

To change the default retry strategy for a single API call, simply provide a `RetryConfig` object to the call:
```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared
from codatsyncexpenses.utils import BackoffStrategy, RetryConfig

with CodatSyncExpenses(
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.companies.create(request={
        "name": "Technicalium",
    },
        RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False))

    if res is not None:
        # handle response
        pass

```

If you'd like to override the default retry strategy for all operations that support retries, you can use the `retry_config` optional parameter when initializing the SDK:
```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared
from codatsyncexpenses.utils import BackoffStrategy, RetryConfig

with CodatSyncExpenses(
    retry_config=RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False),
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.companies.create(request={
        "name": "Technicalium",
    })

    if res is not None:
        # handle response
        pass

```
<!-- End Retries [retries] -->

<!-- Start Error Handling [errors] -->
## Error Handling

Handling errors in this SDK should largely match your expectations. All operations return a response object or raise an exception.

By default, an API error will raise a errors.SDKError exception, which has the following properties:

| Property        | Type             | Description           |
|-----------------|------------------|-----------------------|
| `.status_code`  | *int*            | The HTTP status code  |
| `.message`      | *str*            | The error message     |
| `.raw_response` | *httpx.Response* | The raw HTTP response |
| `.body`         | *str*            | The response content  |

When custom error responses are specified for an operation, the SDK may also raise their associated exceptions. You can refer to respective *Errors* tables in SDK docs for more details on possible exception types for each operation. For example, the `create_async` method may raise the following exceptions:

| Error Type          | Status Code                       | Content Type     |
| ------------------- | --------------------------------- | ---------------- |
| errors.ErrorMessage | 400, 401, 402, 403, 429, 500, 503 | application/json |
| errors.SDKError     | 4XX, 5XX                          | \*/\*            |

### Example

```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import errors, shared

with CodatSyncExpenses(
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = None
    try:
        res = s.companies.create(request={
            "name": "Technicalium",
        })

        if res is not None:
            # handle response
            pass

    except errors.ErrorMessage as e:
        # handle e.data: errors.ErrorMessageData
        raise(e)
    except errors.SDKError as e:
        # handle exception
        raise(e)
```
<!-- End Error Handling [errors] -->

<!-- Start Server Selection [server] -->
## Server Selection

### Override Server URL Per-Client

The default server can also be overridden globally by passing a URL to the `server_url: str` optional parameter when initializing the SDK client instance. For example:
```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared

with CodatSyncExpenses(
    server_url="https://api.codat.io",
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.companies.create(request={
        "name": "Technicalium",
    })

    if res is not None:
        # handle response
        pass

```
<!-- End Server Selection [server] -->

<!-- Start Custom HTTP Client [http-client] -->
## Custom HTTP Client

The Python SDK makes API calls using the [httpx](https://www.python-httpx.org/) HTTP library.  In order to provide a convenient way to configure timeouts, cookies, proxies, custom headers, and other low-level configuration, you can initialize the SDK client with your own HTTP client instance.
Depending on whether you are using the sync or async version of the SDK, you can pass an instance of `HttpClient` or `AsyncHttpClient` respectively, which are Protocol's ensuring that the client has the necessary methods to make API calls.
This allows you to wrap the client with your own custom logic, such as adding custom headers, logging, or error handling, or you can just pass an instance of `httpx.Client` or `httpx.AsyncClient` directly.

For example, you could specify a header for every request that this sdk makes as follows:
```python
from codat_sync_for_expenses import CodatSyncExpenses
import httpx

http_client = httpx.Client(headers={"x-custom-header": "someValue"})
s = CodatSyncExpenses(client=http_client)
```

or you could wrap the client with your own custom logic:
```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.httpclient import AsyncHttpClient
import httpx

class CustomClient(AsyncHttpClient):
    client: AsyncHttpClient

    def __init__(self, client: AsyncHttpClient):
        self.client = client

    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        request.headers["Client-Level-Header"] = "added by client"

        return await self.client.send(
            request, stream=stream, auth=auth, follow_redirects=follow_redirects
        )

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        return self.client.build_request(
            method,
            url,
            content=content,
            data=data,
            files=files,
            json=json,
            params=params,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            extensions=extensions,
        )

s = CodatSyncExpenses(async_client=CustomClient(httpx.AsyncClient()))
```
<!-- End Custom HTTP Client [http-client] -->

<!-- Start Authentication [security] -->
## Authentication

### Per-Client Security Schemes

This SDK supports the following security scheme globally:

| Name          | Type   | Scheme  |
| ------------- | ------ | ------- |
| `auth_header` | apiKey | API key |

You can set the security parameters through the `security` optional parameter when initializing the SDK client instance. For example:
```python
from codat_sync_for_expenses import CodatSyncExpenses
from codat_sync_for_expenses.models import shared

with CodatSyncExpenses(
    security=shared.Security(
        auth_header="Basic BASE_64_ENCODED(API_KEY)",
    ),
) as s:
    res = s.companies.create(request={
        "name": "Technicalium",
    })

    if res is not None:
        # handle response
        pass

```
<!-- End Authentication [security] -->

<!-- Start Debugging [debug] -->
## Debugging

You can setup your SDK to emit debug logs for SDK requests and responses.

You can pass your own logger class directly into your SDK.
```python
from codat_sync_for_expenses import CodatSyncExpenses
import logging

logging.basicConfig(level=logging.DEBUG)
s = CodatSyncExpenses(debug_logger=logging.getLogger("codat_sync_for_expenses"))
```
<!-- End Debugging [debug] -->

<!-- Placeholder for Future Speakeasy SDK Sections -->

<!-- Start Codat Support Notes -->
## Support

If you encounter any challenges while utilizing our SDKs, please don't hesitate to reach out for assistance. 
You can raise any issues by contacting your dedicated Codat representative or reaching out to our [support team](https://github.com/codatio/client-sdk-python/blob/master/mailto:support@codat.io).
We're here to help ensure a smooth experience for you.
<!-- End Codat Support Notes -->

<!-- Start Codat Generated By -->
### Library generated by [Speakeasy](https://docs.speakeasyapi.dev/docs/using-speakeasy/client-sdks)
<!-- End Codat Generated By -->